/*
 * Copyright (c) 2000, Amnon BARAK (amnon@cs.huji.ac.il). All rights reserved.
 *
 * Permission to use, copy and distribute this software is hereby granted 
 * under the terms of version 2 or any later version of the GNU General Public
 * License, as published by the Free Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED IN ITS "AS IS" CONDITION, WITH NO WARRANTY
 * WHATSOEVER. NO LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
 * FROM THE USE OF THIS SOFTWARE WILL BE ACCEPTED.
 */

/*
 * Author(s): Ariel Eizenberg
 */

#ifndef __MAPFILE_H
#define __MAPFILE_H

#include <linux/mosctl.h>
#include <sys/types.h>
#include <time.h>
#include <stdio.h>


typedef int node_t;

struct mosixmap;
struct mosixmap_iter;

struct mosixmap_range
{
  unsigned char ip[4];
  node_t firstnum, lastnum;
  int length;
};

struct mosixmap_node
{
  unsigned char ip[4];
  node_t nodenum;
};

struct mosixmap_alias
{
  unsigned char ip[4];
  int nodenum;
};

typedef int (*iter_function)(unsigned char *ip,int hostflags,void* data);

/*########## Create & Load & Save & Free  ########## */

/* Create a new map (malloc it) */
int mosmap_new(struct mosixmap** map);

/* Load map from specified file */
int mosmap_load(struct mosixmap** map, const char* path);

/* Saves map to the specified file */
int mosmap_save(struct mosixmap* map, const char* path);

/* Load map from specified stream (closes it when done) */
int mosmap_load_stream(struct mosixmap** map, FILE* stream);

/* Saves map to the specified stream(does not close it when done) */
int mosmap_save_stream(struct mosixmap* map, FILE* stream);

/* Load a map from a mosixnet stream (i.e. /proc/mosix/admin/config) */
int mosmap_load_mosnet(struct mosixmap** _map, const char* path);

/* Save a map to a mosixnet stream (i.e. /proc/mosix/admin/config) */
int mosmap_save_mosnet(struct mosixmap* map, const char* path);

/* Load a map from a char buffer */
int mosmap_load_buffer(struct mosixmap** _map, char* buffer, int len);

/* Save a map to a char buffer (allocated by the func */
int mosmap_save_buffer(struct mosixmap* map, char** buffer, int* len);

/* Frees memory taken by specified map */
int mosmap_free(struct mosixmap** map);


/*########## Load from network ########## */

/* Load from specified server */
int mosmap_load_net(struct mosixmap** map, const char* server, struct timeval* timeout);

/* Load from broadcast using interface 'iface' (iface==NULL => iface="eth0") */
int mosmap_load_netbcast(struct mosixmap** map, const char* iface, struct timeval* timeout);

/*########## Add & Remove Node ########## */

/* Adds node to the map */
int mosmap_addnode(struct mosixmap* map, node_t nodenum, const unsigned char ip[4]);

/* Remove node from map */
int mosmap_removenode(struct mosixmap* map, node_t nodenum);

/* Remove node from map by ip */
int mosmap_removenodeip(struct mosixmap* map, const unsigned char ip[4]);


/*########## Add & Remove Alias ########## */

/* Adds a node alias to the map */
int mosmap_addalias(struct mosixmap* map, node_t nodenum, const unsigned char ip[4]);

/* Remove alias from map */
int mosmap_removealias(struct mosixmap* map, const unsigned char ip[4]);


/*########## Add & Remove Range ########## */

/* Add a node range to the map */
int mosmap_addrange(struct mosixmap* map, node_t nodenum, const unsigned char ip[4], int length);

/* Remove a node range from the map */
int mosmap_removerange(struct mosixmap* map, node_t nodenum, int length);

/* Remove a node range from the map by ip */
int mosmap_removerangeip(struct mosixmap* map, const unsigned char ip[4], int length);


/*########## Get Aliases ########## */

/* Get a node's aliases */
int mosmap_getaliases(struct mosixmap* map, int nodenum, struct mosixmap_alias** aliases, int* length);

/* Get all the aliases in the map */
int mosmap_getallaliases(struct mosixmap* map, struct mosixmap_alias** aliases, int* length);


/*########## Iteration ########## */

/* Begin iteration of the map */
int mosmap_iter_init(struct mosixmap* map, struct mosixmap_iter** iter);

/* Get next node range from the map */
int mosmap_iter_next_range(struct mosixmap_iter* iter, struct mosixmap_range* range);

/* Get next node from the map */
int mosmap_iter_next_node(struct mosixmap_iter* iter, struct mosixmap_node* node);

/* Free memory used by iteration */
int mosmap_iter_done(struct mosixmap_iter** iter);


/*########## {Host,IP) <-> {Node}  ########## */

/* Get the specified ip's node number */
int mosmap_gethostnode(struct mosixmap* map,const unsigned char ip[4], node_t* nodenum);

/* Get the ip of the specified node */
int mosmap_getnodehost(struct mosixmap* map, node_t nodenum, unsigned char ip[4]);

/* Get the specified host's node number */
int mosmap_getnamenode(struct mosixmap* map, char* name, node_t* nodenum);

/* Get the host of the specified node */
int mosmap_getnodename(struct mosixmap* map, node_t nodenum, char* name, int *namelen);

/*########## struct mosixnet*  ########## */

/* Allocate a mosixnet array */
int mosmap_net_alloc(int size, struct mosixnet** net);

/* Dynmicallay expand a mosixnet array */
int mosmap_net_expand(struct mosixnet **net, int *size);

/* Convert a mosixmap into a mosixnet */
int mosmap_net2map(struct mosixnet* net, int size, struct mosixmap** map);

/* Convert a mosixnet into a mosixmap */
int mosmap_map2net(struct mosixmap* map, struct mosixnet** net, int *size);

/*########## error handling  ########## */

/* Print the last error to std error */
void mosmap_perror(unsigned const char* msg);

/* Get the last error */
const char* mosmap_strerror();

/* Last error number */
extern int mosmap_errorno;
int mosmap_libcerrno;

#define MOSMAP_NOERROR            0   /* Success */
#define MOSMAP_NULLMAP           -1   /* Null map pointer */
#define MOSMAP_MEMERR            -2   /* Memory allocation error */
#define MOSMAP_INVALIDFILE       -3   /* Invalid file path */
#define MOSMAP_INVALIDFORMAT     -4   /* Invalid map file format */
#define MOSMAP_IOERR             -5   /* IO Error */
#define MOSMAP_UNKNOWNHOST       -6   /* Error resolving host name */
#define MOSMAP_DUPLICATENODES    -7   /* Duplicate node number */
#define MOSMAP_DUPLICATEIP       -8   /* Duplicate IP */
#define MOSMAP_RANGETOOLONG      -9   /* Range exceeds IP boundary */
#define MOSMAP_NOALIASES        -10   /* Node does not have any aliases */
#define MOSMAP_NOSUCHNODE       -11   /* No such node */
#define MOSMAP_NOSUCHIP         -12   /* No such IP */
#define MOSMAP_NOSUCHALIAS      -13   /* No such alias */
#define MOSMAP_ALIASONLYIP      -14   /* Cannot remove node as alias */
#define MOSMAP_NOMORELMS        -15   /* Iteration done - no more elements */
#define MOSMAP_TBLOVERFLOW      -16   /* MOSIX net table overflow */
#define MOSMAP_NOPIPE           -17   /* Failed in pipe() */
#define MOSMAP_CANTGETBCASTADDR -18   /* Failed to get broadcast address */
#define MOSMAP_TIMEOUT          -19   /* Network timeout */
#define MOSMAP_NOMAPSERV        -20   /* No entry for mapserv in /etc/services */

#define MOSMAP_LASTERROR        -20

#endif
